GroupedUITableViews
==========================

UITableViews are ubiquitous  in almost any iOS app. Grouped table views show information in distinct sections, making it a clean visual interface for many different cases. 

Grouped table views though are not always as trivial to implement and maintain. The problem is that, depending of individual sections complexity, trying to handle all of their (often unrelated) logic in a single ViewController can get quite messy. And when you're finally done with the switch statements, some arrays of arrays of dictionaries, custom cells, various section logic handlers, etc. -- things are far from over. Just wait till you need to come back in a few weeks to fix a bug or implement an additional feature...
 
One of my app, [iCasualTours](http://www.icasualtours.com/), is a heavy grouped UITableViews addict. As the app grew larger over time, it became obvious a cleaner approach is needed.  

The main idea of GroupedUITableViews was in moving the logic of individual sections to small dedicated controller classes. The main ViewController now just dispatches its TableView Data Source / Delegates methods to these classes and let them handle whatever needs to happen on individual sections levels. While there is some initial work creating the sections controllers, when things are set up everything becomes simple and real easy to maintain. 

GroupedUITableViews are based on three main classes. ````GroupedTableViewController```` is the UITableViewController that dispatches its table view methods. ````GroupedTableUIController```` is the dispatcher class that simply keeps the register of specific sections and delivers messages to them. ````GroupedTableSectionController```` class represents a specific section in a grouped UITableView, implementing relevant table view methods and encapsulating all of its section-related controller logic.


# Sample Project
The main purpose of the sample project was to show usage of GroupedUITableViews. It consists of a table view with several unrelated sections. The individual sections logic sits in small dedicated section controllers classes and should be farily easy to understand.

# Usage 
A structured approach using code from the sample project:
* Inherit your UITableViewController from ````GroupedTableViewController```` 
* Create a UI Controller class inheriting from ````GroupedTableUIController````. That will be the dispatching class for your view controller. 
* In your UI Contoller, define the table view structure. One way to do it is via using C enums:

````
// defines the TableView Sections
enum SampleViewControllerSections {
    kSampleViewControllerSimpleSection = 0,
    kSampleViewControllerExpandableSection,
    kSampleViewControllerTickerSection,
    kSampleViewControllerStartStopTickingButtonSection,
    kSampleViewControllerNumSections
};
- (NSInteger)numberOfSections {
    return kSampleViewControllerNumSections;
}
````
* Now create your section controllers, each inheriting from ````GroupedTableSectionController````. 
* In the UI Controller, register the section controllers via implementing ````- (void)buildSectionsDispatchTable;```  method:

````
// Builds the dispatch table for SampleViewController
- (void)buildSectionsDispatchTable {
    self.grandDispatchTable = @{
        @(kSampleViewControllerSimpleSection):[[SampleViewControllerSimpleSection alloc]
                                  initWithParentViewController:self.parentVC
                                  SectionID:kSampleViewControllerSimpleSection
                                  DefaultCellID:@"SampleViewControllerSimpleSectionCellID"],
        
        @(kSampleViewControllerExpandableSection):[[SampleViewControllerSuperPowersSection alloc]
                                  initWithParentViewController:self.parentVC
                                  SectionID:kSampleViewControllerExpandableSection
                                  DefaultCellID:@"SampleViewControllerExpandableSectionCellID"],
       ...
````
* Register the UI Controller in your UITableViewController.  That is going to be all of you  table view sections controller code you ever need to write there. :)

````
- (void)viewDidLoad {
    [super viewDidLoad];

    self.title = @"GroupedTableViews Sample";
    
    // specific UI controller, dispatching TableView messages to individual section controllers
    self.uiController = [[SampleViewControllerUIController alloc] initWithParentViewController:self];
}
````
* So much for the initial setup, you can now focus on your individual sections controllers code that is going to sit in small dedicated section controller classes. 

# Compatibility
GroupedUITableViews requires ARC and was optimised for iOS6 and above.
